static char *rcsID = "$Id: sv.c,v 1.3 1997/11/27 08:43:09 sano Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/keysym.h>
#include <X11/Xresource.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Shell.h>
#include <X11/Xaw/Cardinals.h>
#include "lib/Ghostview.h"
#include "lib/ps.h"

char            *progname;
XtAppContext     appCon;
Widget           toplevel, form, page;
struct document *doc;
FILE            *file;
int              curpage = -1;
int              verbose, debug;
  
void jumpPage(p)
{
  if (p < 0)
    p = 0;
  if (p >= doc->numpages)
    p = doc->numpages - 1;
  if (curpage == p)
    return;
  curpage = p;
  if (doc->pageorder == DESCEND)
    p = doc->numpages - p - 1;
  if (GhostviewIsInterpreterReady(page))
    GhostviewNextPage(page);
  else {
    GhostviewEnableInterpreter(page);
    GhostviewSendPS(page, file, doc->beginprolog, doc->lenprolog, False);
    GhostviewSendPS(page, file, doc->beginsetup, doc->lensetup, False);
  }
  if (verbose)
    fprintf(stderr, "Page:%d pos:%d len:%d\n",
	    curpage, doc->pages[p].begin, doc->pages[p].len);
  GhostviewSendPS(page, file, doc->pages[p].begin, doc->pages[p].len, False);
}

#define nextPage()  jumpPage(curpage+1)
#define prevPage()  jumpPage(curpage-1)
#define beginPage() jumpPage(0)
#define endPage()   jumpPage(doc->numpages - 1)

void exitProgram(int code)
{
  GhostviewDisableInterpreter(page);
  XtDestroyApplicationContext(appCon);
  exit(code);
}

void keyin(Widget w, XtPointer clientData, XEvent *e, Boolean *b)
{
  int key;
  if (e->type == KeyPress) {
    switch (key = XLookupKeysym((XKeyEvent *)e, 0)) {
    case XK_q:
    case XK_Q:
      exitProgram(0);

    case XK_Page_Up:
    case XK_Up:
    case XK_BackSpace:
      prevPage();
      break;

    case XK_Page_Down:
    case XK_Down:
    case XK_Return:
    case ' ':
      nextPage();
      break;

    case XK_Home:
      beginPage();
      break;

    case XK_End:
      endPage();
      break;

    case '1': case '2': case '3': case '4':
    case '5': case '6': case '7': case '8': case '9':
      jumpPage(key-'1');
      break;

    default:
      if (verbose)
	printf("%d\n", key);
    }
  }
}

void getDisplaySize(Display *dpy, int *w, int *h)
{
  XWindowAttributes xwattr;
  Window rootWin;

  rootWin = DefaultRootWindow(dpy);
  XGetWindowAttributes(dpy, rootWin, &xwattr);
  *w = xwattr.width;
  *h = xwattr.height;
}

extern char *optarg;
extern int optind;
extern int optopt;

int x, y, zoomx, zoomy, vW, vH, wW, wH;

void usage()
{
  fprintf(stderr, "Usage: %s [-v] [-z zoom] psfile\n", progname);
  exit(1);
}

void main(int argc, char **argv)
{
  char *filename;
  int ch, llx, lly, urx, ury, t;
  int portrait;

  if ((progname = strrchr(argv[0], '/')) != NULL)
    progname++;
  else
    progname = argv[0];

  toplevel = XtVaAppInitialize(&appCon, "Sv", NULL, 0, &argc, argv, NULL, NULL);

  while ((ch = getopt(argc, argv, "vdz:D:")) != EOF) switch (ch) {

  case 'v':
    verbose = 1;
    break;

  case 'd':
    debug = 1;
    break;

  case 'z':
    if (sscanf(optarg, "%dx%d\n", &zoomx, &zoomy) != 2)
      zoomx = zoomy = atoi(optarg);
    if (zoomx <= 0 || zoomx > 500)
      zoomx = 100;
    if (zoomy <= 0 || zoomy > 500)
      zoomy = 100;
    break;

  case 'D':
    {
      int w, h;
      if (sscanf(optarg, "%dx%d\n", &w, &h) == 2)
	wW = w, wH = h;
      /*printf("%d,%d\n", wW, wH); */
    }
    break;

  default:
    usage();
  }

  if (optind != argc - 1)
    usage();

  filename = argv[optind];

  if ((file = fopen(filename, "r")) == NULL) {
    perror(filename);
    exit(1);
  }

  if (!(doc = psscan(file))) {
    fprintf(stderr, "%s: %s: cannot get psfile\n", progname, doc);
    exit(1);
  }
  if (doc->numpages == 0) {
    fprintf(stderr, "%s: %s: cannot found page\n", progname, doc);
    exit(1);
  }

  if (verbose)
    for (curpage = 0; curpage < doc->numpages; curpage++)
      printf("[%d] %d %d\n", curpage, doc->pages[curpage].begin, doc->pages[curpage].len);

#ifdef OLD
  x = 302, y = 570;
/*  vW = 533, vH = 400; */
  vW = 520, vH = 388 ; 
  portrait = 1;
#else
  portrait = 0;
/*  vW = 842, vH = 592;  */
  vW = 11*72, vH = (int)(8.5*72); 
  vW = 842, vH = 595;  
  x = vW/2, y = vH/2;
#endif

  if (wW == 0 || wH == 0) 
    getDisplaySize(XtDisplay(toplevel), &wW, &wH);

  if (zoomx == 0 || zoomy == 0)
    zoomx = zoomy = 100;

  t = (vW * wH) / wW;
  if (t < wH)
    vH = t;
  else 
    vW = (vH * wW) / wH;

  if (zoomx != 100 || zoomy != 100) {
    vW = (vW * 100) / zoomx;
    vH = (vH * 100) / zoomy;
   }

  if (portrait) {
    llx = x - vW/2;
    lly = y - vH/2;
    urx = x + vW/2;
    ury = y + vH/2;
  } else {
    llx = y - vH/2;
    lly = x - vW/2;
    urx = y + vH/2;
    ury = x + vW/2;
  }

  if (verbose || debug) {
    fprintf(stderr, "(wW,wH)=(%d,%d) ",   wW, wH);
    fprintf(stderr, "(vW,vH)=(%d,%d) ",   vW, vH);
    fprintf(stderr, "(llx,lly)=(%d,%d) ", llx, lly);
    fprintf(stderr, "(urx,ury)=(%d,%d) ", urx, ury);
    fprintf(stderr, "(zoomx,zoomy)=(%d,%d)\n", zoomx, zoomy);



  }

  form = XtVaCreateManagedWidget("Sheet", boxWidgetClass, toplevel, NULL);

  page = XtVaCreateManagedWidget("page", ghostviewWidgetClass, form, 
				 XtNllx,    llx,
				 XtNlly,    lly,
				 XtNurx,    urx,
				 XtNury,    ury,
				 XtNwidth,  wW,
				 XtNheight, wH,
				 XtNborderWidth, 0,
				 XtNorientation, portrait?XtPageOrientationPortrait:XtPageOrientationLandscape,
/*				 XtNfilename, "out.ps", */
				 NULL);

  XtRealizeWidget(toplevel);
  XtAddEventHandler(page, KeyPressMask, FALSE, keyin, NULL);

  beginPage();

  XtAppMainLoop(appCon);

}

